#include <complex>
#include <complex.h>

#ifdef USE_LAPACK
#ifdef USE_MKL
#include <mkl_lapacke.h>
#else
#include <lapacke.h>
#endif
#ifndef INCLUDE_LAPACK_CALLS_H_
#include "../include/lapack_calls.h"
#endif
#endif

void zinvert(std::complex<double> **mat, lapack_int n, int &jer) {
#ifdef USE_LAPACK
  jer = 0;
  __complex__ double *arr = new __complex__ double[n * n];
  const __complex__ double uim = 1.0*I;
#ifdef USE_MKL
  MKL_Complex16 *arr2 = (MKL_Complex16 *) arr;
#endif
  for (lapack_int i = 0; i < n; i++) {
    for (lapack_int j = 0; j < n; j++) {
      lapack_int idx = i + n * j;
      arr[idx] = mat[j][i].real() + uim * mat[j][i].imag();
    }
  }
  
  lapack_int* IPIV = new lapack_int[n]();
  
#ifdef USE_MKL
  LAPACKE_zgetrf(LAPACK_ROW_MAJOR, n, n, arr2, n, IPIV);
  LAPACKE_zgetri(LAPACK_ROW_MAJOR, n, arr2, n, IPIV);
#else
  LAPACKE_zgetrf(LAPACK_ROW_MAJOR, n, n, arr, n, IPIV);
  LAPACKE_zgetri(LAPACK_ROW_MAJOR, n, arr, n, IPIV);
#endif
  for (lapack_int i = 0; i < n; i++) {
    for (lapack_int j = 0; j < n; j++) {
      lapack_int idx = i + n * j;
      mat[j][i] = std::complex<double>(creal(arr[idx]), cimag(arr[idx]));
    }
  }
  delete[] IPIV;
  delete[] arr;
#endif
}
